function out = pfBCC( filein, fileout, gridType, filterType )
% Prefilters data for either tricubic B-spline interpolation on CC or
% Quintic box spline interpolation on BCC.
%
% Usage:
% out = pfBCC(filein, fileout, gridType)
%
% where: filein specifies the input file
%             fileout specified the name of the output file 
%            gridType is one of 'CC' or 'BCC'
%            filterType is one of 'I' or 'Q', specifying either
%            interpolation or quasi-interpolation
%
% out returns the number of elements written
% Data is written in single precision floating point vud format.

% Read data, make the filter, perform filtering, and write the data

switch lower(gridType)
    case 'cc'
        va = readvudBCC(filein);
        if( lower(filterType) == 'i' )
            f = [1/6 2/3 1/6];
            cf = makeFilterCC(tpFilter(f,f,f), size(va,2), size(va,1), size(va,3));
            va = real(ifftn(fftn(va) ./ fftn(cf)));
        elseif( lower(filterType) == 'q')
            fn = [1/362880, 251/181440, 913/22680, 44117/181440, 15619/36288, 44117/181440, 913/22680, 251/181440, 1/362880];
            fd = [1/120, 13/60, 11/20, 13/60, 1/120];
            fac = [1/5040, 1/42, 397/1680, 151/315, 397/1680, 1/42, 1/5040];
            cfn = makeFilterCC(tpFilter(fn,fn,fn), size(va,2), size(va,1), size(va,3));
            cfd = makeFilterCC(tpFilter(fd,fd,fd), size(va,2), size(va,1), size(va,3));
            cfac =makeFilterCC(tpFilter(fac,fac,fac), size(va,2), size(va,1), size(va,3));
            va = ifftn(fftn(va) .* fftn(cfn) ./ (fftn(cfd).*fftn(cfac)));
            va = real(va);
        else
           error('Wrong filter type, please specify either I or Q'); 
        end
        out = writevudBCC(fileout, va);
    case 'bcc'
        [va vb] = readvudBCC(filein);
        if( lower(filterType) == 'i')
            [filta, filtb] = readFilter('SSQbs', size(va,2), size(va,1), size(va,3));
            [va vb] = ifft3bcc(fft3bcc(va, vb) ./ fft3bcc(filta, filtb) );
            va = real(va);
            vb = real(vb);
        elseif( lower(filterType) == 'q')
            [nfilta, nfiltb] = readFilter('SSSbs',size(va,2), size(va,1), size(va,3));
            [dfilta, dfiltb] = readFilter('SSNbs',size(va,2), size(va,1), size(va,3));
            [Afilta, Afiltb] = readFilter('ACQbs',size(va,2), size(va,1), size(va,3));
            [va vb] = ifft3bcc(fft3bcc(va, vb) .*( fft3bcc(nfilta, nfiltb) ./ ...
                (fft3bcc(dfilta,dfiltb) .* fft3bcc(Afilta, Afiltb)) ));
            va = real(va);
            vb = real(vb);
        end
        out = writevudBCC(fileout, va,vb);
end

return
